ORDER_CREATE_ENDPOINT = '/b2b/v1/orders/create'
ORDER_UPDATE_ENDPOINT = '/b2b/v1/orders/update'
ORDER_CANCEL_ENDPOINT = '/b2b/v1/orders/cancel'
ORDER_DETAIL_ENDPOINT = '/b2b/v1/orders/get'
ORDERS_STATE_ENDPOINT = '/b2b/v1/orders/state'
ORDERS_EVENTS_QUERY_ENDPOINT = '/b2b/v1/orders/events/query'

RECEIPTS_GET_ENDPOINT = '/b2b/v1/receipts/get'
RECEIPTS_UPLOAD_ENDPOINT = '/b2b/v1/receipts/documents/upload'

WMS_PICKING_SET_STATE_ENDPOINT = '/b2b/v1/wms/picking/set-state'
LOGISTIC_DELIVERY_SET_STATE_ENDPOINT = '/b2b/v1/logistics/delivery/set-state'

PRICE_GET_ENDPOINT = '/b2b/v1/prices/get'
PRICE_SET_ENDPOINT = '/b2b/v1/prices/set'

PRICE_LIST_GET_ENDPOINT = '/b2b/v1/pricelists/get'
PRICE_LIST_CREATE_ENDPOINT = '/b2b/v1/pricelists/create'
PRICE_LIST_UPDATES_ENDPOINT = '/b2b/v1/pricelists/query'

PRODUCT_UPDATES_ENDPOINT = '/b2b/v1/products/query'
PRODUCT_CREATE_ENDPOINT = '/b2b/v1/products/create'
PRODUCT_MEDIA_CREATE_ENDPOINT = '/b2b/v1/products/media/create'

PRODUCT_VAT_GET_ENDPOINT = '/b2b/v1/products-vat/get'
PRODUCT_VAT_UPDATE_ENDPOINT = '/b2b/v1/products-vat/update'
PRODUCT_VAT_CREATE_ENDPOINT = '/b2b/v1/products-vat/create'

STORE_PRICE_LIST_LINK_CREATE_ENDPOINT = '/b2b/v1/store-pricelist-links/create'
STORE_PRICE_LIST_LINK_GET_ENDPOINT = '/b2b/v1/store-pricelist-links/get'

DISCOUNTS_CREATE_ENDPOINT = '/b2b/v1/discounts/create'

STOCK_UPDATE_ENDPOINT = '/b2b/v1/stocks/update'
STOCK_INITIALIZE_ENDPOINT = '/b2b/v1/stocks/initialize'
STOCK_GET_ENDPOINT = '/b2b/v1/stocks/query'

STORES_GET_ENDPOINT = '/b2b/v1/stores/get'
